'' **************************
'' ** SD INTERFACE TRAINER **
'' ** By Jeff Ledger       **
'' **************************

'' This module is intended for use with the SD COOKBOOK ENTRY
'' This uses FullDuplexSerialPlus to interact with the Propeller

'' Data displayed on 30,31 (Programming port)
'' Baud rate should be: 57600

'' RR20090331   change pin nos & baudrate to 115200 for TriBladeProp Blade #2
''              Notes: TestMountSD does not appear to work (locks) but the
''                     microSD was correctly written and read and verified on a PC.
''                     The EEPROM is required for this program to function.

CON
   
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
   
  '' SD Configuration
  
   spiDO     = 9  '16   ''************************************
   spiClk    = 28 '17   ''** This configuration option must **
   spiDI     = 8  '18   ''** be included in your code.      ** 
   spiCS     = 14 '19   ''************************************
   
OBJ
   
  debug: "FullDuplexSerialPlus"   ''Used for interaction with the Propeller
  sd:    "fsrwFemto"              ''SD Software used in FemtoBASIC


VAR

  long ioControl[2] ''** Variable required for SD Routines **    
    
       
PUB Main | r,data

'  debug.start(31,30,0,57600)   ''Open serial communication with PST
  debug.start(31,30,0,115200)   ''Open serial communication with PST
  waitcnt(clkfreq*2 + cnt)
  waitcnt(clkfreq*2 + cnt)
  debug.tx(Debug#CLS)

  ''TestMountSD  '' Uncomment this line to test your SD setup. 
  ''TestSerial   '' Uncomment this line to test your communication setup.

  debug.str(string("Running the propeller SD training program",13,13))   

  ''repeat                        '<==================stop here

''****************************************************************************
  
  sd.start(@ioControl) '' ** Start the SD Routines **

  
  '' Send a line of text to the SD

  debug.str(string(13,"** Sending information to SD",13))
  sd.mount(spiDO,spiClk,spiDI,spiCS) '' Mount the SD
  sd.popen(string("alpha.txt"),"w")  '' Open alpha.txt for write
  sd.str(string("This is a line of text written to the SD card.",13,10))
  sd.str(string("This is another line of text written to the SD card.",13,10))
  sd.pclose  '' Close the file
  sd.unmount '' Unmount the SD

  
  '' Read the data back to the terminal 

  debug.str(string(13,"** Reading information back from SD",13))
  sd.mount(spiDO,spiClk,spiDI,spiCS)     '' Mount the SD
  r := sd.popen(string("alpha.txt"),"r") '' Open alpha.txt for read 
    repeat until r == -1 '' Start a loop until the end of the file is reached.
         r := sd.pgetc   '' Grap a single character
         if r > 0        '' If character is greater than nothing, display it.
             debug.tx(r)
  sd.unmount '' Unmount the SD

  
  debug.str(string(13,"** Routine Finished",13))

  
PUB TestMountSD   ''SD Connection Test Routine
  
  if \sd.mount(spiDO,spiClk,spiDI,spiCS) < 0          ''    SD Connection Startup
     debug.str(string(13,"No SD card present.",13))   ''*****************************
     debug.str(string("SD card expected on: P"))      ''** This SD startup routine **
     debug.dec(spiDO)                                 ''** provides a gracefull    **
     debug.str(string(", P"))                         ''** exit if something is    **
     debug.dec(spiClk)                                ''** not connected properly  ** 
     debug.str(string(", P"))                         ''** or if the card is       **
     debug.dec(spiDI)                                 ''** unplugged.              **
     debug.str(string(", P"))                         ''**                         **
     debug.dec(spiCS)                                 ''** Code adapted from:      **
     debug.str(string(13,13,"Program halted.",13))    ''** Mike Green              **
     abort                                            ''*****************************
  else
     debug.str(string(13,"Connection to SD correctly established..",13))
     sd.unmount
     
PUB TestSerial | counter  ''Simple communication test routine

  ''Send messates stored in the DAT block.
  counter := 1
  repeat
     Debug.Str(string("This is a line number: "))
     Debug.Dec(counter++)
     Debug.Str(string(13,"PST configuration is working.",13,13))
     waitcnt(clkfreq + cnt)